@@ -1,3 +1,4 @@
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 #include <config.h>
 #include "printer-properties.h"
 
@@ -17,6 +18,7 @@
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenuitem.h>
 #include <gtk/gtkmessagedialog.h>
+#include <gtk/gtknotebook.h>
 #include <gtk/gtkoptionmenu.h>
 #include <gtk/gtkscrolledwindow.h>
 #include <gtk/gtksizegroup.h>
@@ -30,8 +32,10 @@
 #include <libgnomecups/gnome-cups-request.h>
 #include <libgnomecups/gnome-cups-printer.h>
 #include <libgnomecups/gnome-cups-queue.h>
+#include <libgnomecups/gnome-cups-util.h>
 #include <libgnomecups/gnome-cups-ui-print.h>
 #include <libgnomecups/gnome-cups-ui-util.h>
+#include <libgnomecups/gnome-cups-ui-driver.h>
 
 #include <cups/cups.h>
 
@@ -40,6 +44,12 @@ typedef struct {
 	char *value;
 } PrinterInfoData;
 
+static gboolean
+need_admin_priviledges (void)
+{
+	return (geteuid () != 0);
+}
+
 static void
 unref_group (gpointer user_data, GObject *obj)
 {
@@ -352,10 +362,8 @@ hookup_optional_option_menu_xml (GladeXML *xml,
 				 const char *hide_widget)
 {
 	if (!hookup_option_menu_xml (xml, option_menu_name, key)) {
-		GtkWidget *widget;
-
-		widget = glade_xml_get_widget (xml, hide_widget);
-		gtk_widget_hide (widget);
+		gtk_widget_hide (glade_xml_get_widget (xml, hide_widget));
+		gtk_widget_hide (glade_xml_get_widget (xml, option_menu_name));
 	}
 }
 
@@ -382,6 +390,7 @@ setup_general_page (GladeXML *xml)
 	update_general_page (xml);
 
 	widget = glade_xml_get_widget (xml, "description_entry");
+	if (need_admin_priviledges ())
 	g_signal_connect (widget, "focus_out_event",
 			  G_CALLBACK (description_focus_out_cb), xml);
 
@@ -390,10 +399,6 @@ setup_general_page (GladeXML *xml)
 			  G_CALLBACK (location_focus_out_cb), xml);
 
 	hookup_option_menu_xml (xml, "resolution_option_menu", "Resolution");
-
-	widget = glade_xml_get_widget (xml, "test_page_button");
-	g_signal_connect (widget, "clicked",
-			  G_CALLBACK (test_page_clicked_cb), xml);
 }
 
 static void
@@ -417,9 +422,8 @@ setup_paper_page (GladeXML *xml)
 	hookup_option_menu_xml (xml, "paper_type_option_menu", "MediaType");
 	hookup_option_menu_xml (xml, "source_option_menu", "InputSlot");
 	hookup_option_menu_xml (xml, "double_sided_option_menu", "Duplex");
-
-	hookup_optional_option_menu_xml (xml, "binding_option_menu", "Binding",
-					 "binding_hbox");
+	hookup_optional_option_menu_xml (xml,
+		"binding_option_menu", "Binding", "binding_label");
 }
 
 static void
@@ -568,6 +572,78 @@ setup_advanced_page (GladeXML *xml)
 	gnome_cups_printer_option_list_free (options);
 }
 
+static void
+cb_driver_changed (GCupsDriverSelector *ds, GnomeCupsPrinter *printer)
+{
+	GCupsPPD const *ppd = gcups_driver_selector_get (ds);
+
+	if (ppd != NULL) {
+		ipp_t *request = gnome_cups_request_new_for_printer (
+			CUPS_ADD_PRINTER, gnome_cups_printer_get_name (printer));
+		ippAddString (request, IPP_TAG_PRINTER, IPP_TAG_NAME,
+			"ppd-name", NULL, gnome_cups_strdup (ppd->filename));
+		gnome_cups_request_execute_async (request, NULL, "/admin/", NULL, NULL, NULL);
+	}
+}
+
+static void
+setup_driver_page (GladeXML *xml, GnomeCupsPrinter *printer)
+{
+	GtkWidget  *w, *ds;
+	ppd_file_t *ppd;
+	ppd_attr_t *nickname;
+
+	if (need_admin_priviledges ())
+		return;
+	if (printer == NULL)
+		return;
+	ppd = gnome_cups_printer_get_ppd (printer);
+	if (ppd == NULL)
+		return;
+
+	nickname = ppdFindAttr (ppd, "NickName", NULL);
+	if (nickname != NULL) {
+		ds = gcups_driver_selector_new ();
+		w = glade_xml_get_widget (xml, "prop_notebook");
+		gtk_notebook_append_page (GTK_NOTEBOOK (w),
+			ds, gtk_label_new (_("Driver")));
+		gcups_driver_selector_set_nickname (GCUPS_DRIVER_SELECTOR (ds),
+			nickname->value);
+		gtk_widget_show (ds);
+
+		g_signal_connect_object (ds,
+			"changed",
+			G_CALLBACK (cb_driver_changed), xml, 0);
+	}
+	ppdClose (ppd);
+
+}
+
+#define SU_APP	"gnomesu"
+static void
+cb_become_admin (GnomeCupsPrinter *printer)
+{
+	GError *err = NULL;
+	char *argv[] = { NULL, "gnome-cups-manager", "-p", NULL, NULL };
+	argv[0] = (char *)SU_APP;
+	argv[2] = (char *)gnome_cups_printer_get_name (printer);
+	g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
+		NULL, NULL, NULL, &err);
+
+	if (err != NULL) {
+		GtkWidget *dialog = gtk_message_dialog_new_with_markup (NULL, 
+			 GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL, 
+			 GTK_MESSAGE_ERROR,
+			 GTK_BUTTONS_OK,
+			_("Problems launching %s as root via %s\n%s"),
+			argv[1], SU_APP, err->message);
+		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog); 
+		g_error_free (err);
+	}
+	gtk_main_quit ();
+}
+
 void
 gnome_cups_manager_printer_properties (GnomeCupsPrinter *printer)
 {
@@ -612,14 +688,30 @@ gnome_cups_manager_printer_properties (GnomeCupsPrinter *printer)
 	setup_general_page (xml);
 	setup_paper_page (xml);
 	setup_advanced_page (xml);
+	setup_driver_page (xml, printer);
+
+	button = glade_xml_get_widget (xml, "admin_button");
+	if (need_admin_priviledges ())
+		g_signal_connect_swapped (button,
+			"clicked",
+			G_CALLBACK (cb_become_admin), printer);
+	else
+		gtk_widget_hide (button);
+
+	button = glade_xml_get_widget (xml, "test_page_button");
+	g_signal_connect (button,
+		"clicked",
+		G_CALLBACK (test_page_clicked_cb), xml);
 
 	window = glade_xml_get_widget (xml, "properties_window");
-	g_signal_connect (window, "delete_event",
-			  G_CALLBACK (delete_event_cb), xml);
+	g_signal_connect (window,
+		"delete_event",
+		G_CALLBACK (delete_event_cb), xml);
 
 	button = glade_xml_get_widget (xml, "close_button");
-	g_signal_connect (button, "clicked",
-			  G_CALLBACK (close_clicked_cb), xml);
+	g_signal_connect (button,
+		"clicked",
+		G_CALLBACK (close_clicked_cb), xml);
 
 	gnome_cups_printer_get_icon (printer, &icon_name, NULL);
 	set_window_icon (window, icon_name);
